<?php

namespace Drupal\safedelete\Controller;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\views\Views;
use Drupal\safedelete\AdminHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SearchingOrphanedPageController extends ControllerBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;


  /**
   * SearchingOrphanedPageController constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(Connection $database) {
    //parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->database = $database;
  }

  public static function create(ContainerInterface $container) {
    $database = $container->get('database');
    return new static($database);
  }

  /**
   * Display the markup.
   *
   * @return array
   */
  public function content() {
    if (\Drupal::routeMatch()->getRouteName() == 'safedelete_orphanedpages.content') {
      if (\Drupal::currentUser()->isAuthenticated()) {
        // Was doing this to eliminate admin rendering but template changes should have stripped most of that out.
        // Uncomment the two lines below if we want to force a logout.
        //$session_manager = \Drupal::service('session_manager');
        //$session_manager->delete(\Drupal::currentUser()->id());
      }
    }
    return [
           // '#type' => 'markup',
            '#theme' => 'safedelete_orphanedpages',
            '#orphanedpages_container' => $this->getOrphanedLandingPageOrBasicPage()
    ];
  }


  /**
   * get orphaned pages
   */
  public function getOrphanedLandingPageOrBasicPage () {
    $defconfig = \Drupal::config('safedelete.settings')->get('orphanedpagedefinition');
    $arrayoftypes = [];
    $count = 0;
    $typecount = count($defconfig);

    $query = \Drupal::entityQuery('node');
    $query->condition('type', $defconfig, 'IN');
    $nodeids =  $query->sort('nid', 'ASC')->execute();

    foreach ( $defconfig as $type) {
      if ( strlen($type) > 1 ) {
        $count = $count +1;
        $arrayoftypes[$count] = $type;
      }
    }
    $super_type = \Drupal::config('safedelete.settings')->get('orphanedpagewithoutmenulink');
    $orphanedpages_results = [];
    foreach ($nodeids as $nid ) {
      $uuid = $this->getUuidFromNid($nid);
      $type = $this->getNodeTypeFromNid($nid);
      $resultflag = FALSE;
      $resultreason = "";
      if ($type == $super_type && !AdminHelper::hasMenuLink($nid)) {
        // do further check if the page is linked by other pages
        if ( !AdminHelper::findLinkitReferences($uuid, $nid)) {
          $resultflag = TRUE;
          $resultreason = t('No Menu Link and No Other Pages linked');
          //kint($nid);
        }
      }
      else if ($type != $super_type) {
        // Check if the special page type is linked by other pages
        if ( !AdminHelper::findLinkitReferences($uuid, $nid)) {
          $resultflag = TRUE;
          $resultreason = t('No Other Pages linked to this.');
        }
      }

      if ($resultflag) {
        $langcode = \Drupal::languageManager()->getCurrentLanguage()->getId();
        $entity_manager = \Drupal::entityTypeManager();
        $node = $entity_manager->getStorage('node')->load($nid);
        $ntype =  $node->bundle();
        $ntitle = $node->getTitle();
        $orphanedpages_results[$nid]['nid'] =  $nid;
        $orphanedpages_results[$nid]['title'] = $ntitle;
        $orphanedpages_results[$nid]['href'] =  $this->getUrlForNode($nid, $langcode);
        $orphanedpages_results[$nid]['type'] =  $node->type->entity->label();
        $orphanedpages_results[$nid]['reason'] =  $resultreason;
      }
    } // end of search the orphaned nodes

    return $orphanedpages_results;
  }


  public function getUuidFromNid($nid) {
    $result = $this->database->query('select uuid from node where nid = :node_id', [':node_id' => $nid])
      ->fetchAllAssoc('uuid', \PDO::FETCH_ASSOC);
    $uuid = array_keys($result);
    return reset($uuid);
  }


  public function getNodeTypeFromNid($nid) {
    $result = $this->database->query('select type from node where nid = :node_id', [':node_id' => $nid])
      ->fetchAllAssoc('type', \PDO::FETCH_ASSOC);
    $type = array_keys($result);
    return reset($type);
  }


  public function getUrlForNode($nid, $lang = 'en') {
    $host_path = \Drupal::request()->getSchemeAndHttpHost();
    $base_path = \Drupal::request()->getBasePath();
    if (strlen($base_path) > 0) {
      if (stripos(strval($base_path), '/') < 0) {
        $host_path = $host_path . '/' . $base_path;
      }
      else {
        $host_path = $host_path . $base_path;
      }
    }
    $relative = \Drupal::service('path.alias_storage')->load(['source' => '/node/' . $nid, 'langcode' => $lang]);
    if (isset($relative['alias'])) {
      $url = $host_path . '/' . $lang . $relative['alias'];
    }
    else {
      $url = $host_path . '/' . $lang . '/node/' . $nid;
    }
    return $url;
  }
}
